""" Manages the generation of C++ header files """


def generate_file(filename, tag, content, headerguard=True):
    """
    Write a file from the generation cache:

    Arguments:
    ----------
    filename : str
        The filename to write the generated code to.
    tag: str
        The tag, that all entries related to this file in the cache have.
    content: list
        A list of Generables to put into the file.

    Keyword Arguments:
    ------------------
    headerguard : bool
        Whether a double inclusion protection header should be added to the file.
        The name of the macro is mangled from the absolute path. Defaults to True.
    """
    with open(filename, 'w') as f:
        # Add a double inclusion protection header
        if headerguard:
            macro = filename.upper().replace("/", "_").replace(".", "_")
            f.write("#ifndef {0}\n#define {0}\n\n".format(macro))

        # Add the includes from the cache
        from dune.perftool.generation import retrieve_cache_items
        for inc in retrieve_cache_items((tag, 'include'), union=False):
            from cgen import Include
            assert isinstance(inc, Include)
            for line in inc.generate():
                f.write(line)
            f.write('\n')

        f.write('\n\n')

        for c in content:
            from cgen import Generable
            assert isinstance(c, Generable)
            for line in c.generate():
                f.write(line)

        if headerguard:
            f.write("\n\n#endif //{}\n".format(macro))
